@@ -15,6 +15,7 @@ import subprocess
 import tempfile
 import traceback
 import warnings
+import zlib
 
 try:
     import platform
@@ -1632,6 +1633,7 @@ class RegExSelector:
     def __call__(self, testname, tags=None):
         return self.pattern.search(testname)
 
+
 def string_selector(s):
     ix = s.find(':')
     if ix == -1:
@@ -1639,7 +1641,8 @@ def string_selector(s):
     else:
         return TagsSelector(s[:ix], s[ix+1:])
 
-class ShardExcludeSelector:
+
+class ShardExcludeSelector(object):
     # This is an exclude selector so it can override the (include) selectors.
     # It may not provide uniform distribution (in time or count), but is a
     # determanistic partition of the tests which is important.
@@ -1647,36 +1650,11 @@ class ShardExcludeSelector:
         self.shard_num = shard_num
         self.shard_count = shard_count
 
-    def __call__(self, testname, tags=None):
-        return abs(hash(testname)) % self.shard_count != self.shard_num
-
-
-def refactor_for_py3(distdir, cy3_dir):
-    # need to convert Cython sources first
-    import lib2to3.refactor
-    from distutils.util import copydir_run_2to3
-    with open('2to3-fixers.txt') as f:
-        fixers = [line.strip() for line in f if line.strip()]
-    if not os.path.exists(cy3_dir):
-        os.makedirs(cy3_dir)
-    import distutils.log as dlog
-    dlog.set_threshold(dlog.INFO)
-    copydir_run_2to3(distdir, cy3_dir, fixer_names=fixers,
-                     template = '''
-                     global-exclude *
-                     graft Cython
-                     recursive-exclude Cython *
-                     recursive-include Cython *.py *.pyx *.pxd
-                     recursive-include Cython/Debugger/Tests *
-                     recursive-include Cython/Utility *
-                     recursive-exclude pyximport test
-                     include Tools/*.py
-                     include pyximport/*.py
-                     include runtests.py
-                     include cython.py
-                     include cythonize.py
-                     ''')
-    sys.path.insert(0, cy3_dir)
+    def __call__(self, testname, tags=None, _hash=zlib.crc32, _is_py2=sys.version_info[0] < 3):
+        # Cannot use simple hash() here as shard processes might use different hash seeds.
+        # CRC32 is fast and simple, but might return negative values in Py2.
+        hashval = _hash(testname) & 0x7fffffff if _is_py2 else _hash(testname.encode())
+        return hashval % self.shard_count != self.shard_num
 
 
 class PendingThreadsError(RuntimeError):
